package main

import (
	"context"
	"encoding/json"
	"log/slog"
	"net/http"
	"os"
	"os/signal"
	"strings"
	"syscall"

	"codeberg.org/gbcox/beetpost/mpdpost"
	"codeberg.org/gbcox/beetpost/mprispost"
)

var (
	AppVersionFull  = "dev"
	CommitFull      = "none"
	ShortCommitFull = "none"
)

type Item struct {
	Track         string            `json:"Track"`
	Artist        string            `json:"Artist"`
	Album         string            `json:"Album"`
	ProviderIds   map[string]string `json:"ProviderIds"`
	PlaySessionId string            `json:"PlaySessionId"`
}

type Request struct {
	Event string `json:"Event"`
	User  string `json:"User"`
	Item  Item   `json:"Item"`
}

func initLogging() {
	var level slog.Level
	switch strings.ToLower(os.Getenv("BEETPOST_LOG_LEVEL")) {
	case "debug":
		level = slog.LevelDebug
	case "warn":
		level = slog.LevelWarn
	case "error":
		level = slog.LevelError
	default:
		level = slog.LevelInfo
	}

	handler := slog.NewJSONHandler(os.Stdout, &slog.HandlerOptions{
		Level: level,
	})
	slog.SetDefault(slog.New(handler))

	slog.Info("🎚 Logging level set", "level", level.String())
	slog.Info("📮 Starting Beetpost",
		"version", AppVersionFull,
		"commit", CommitFull,
		"shortCommit", ShortCommitFull)
}

func SendPayload(source, event, user string, item Item) {
	url := os.Getenv("BEETBRAINZ_URL_" + strings.ToUpper(source))
	if url == "" {
		slog.Warn("⛔ No endpoint configured for source", "source", source)
		return
	}

	req := Request{
		Event: event,
		User:  user,
		Item:  item,
	}

	data, err := json.Marshal(req)
	if err != nil {
		slog.Error("💥 Payload marshal failed", "error", err)
		return
	}

	resp, err := http.Post(url, "application/json", strings.NewReader(string(data)))
	if err != nil {
		slog.Warn("📭 Dispatch failed", "source", source, "error", err)
		return
	}
	defer resp.Body.Close()

	slog.Info("📤 Payload posted", "source", source, "track", item.Track, "status", resp.StatusCode)
}

func main() {
	initLogging()

	ctx, cancel := context.WithCancel(context.Background())
	defer cancel()

	mpdEnabled := strings.ToLower(os.Getenv("MPDPOST_ENABLE")) == "true"
	mprisEnabled := strings.ToLower(os.Getenv("MPRISPOST_ENABLE")) == "true"

	if !mpdEnabled && !mprisEnabled {
		slog.Warn("🔇 No towers enabled — exiting")
		os.Exit(0)
	}

	slog.Info("🎙️ Beetpost activated")

	if mpdEnabled {
		slog.Info("🛰 MPDPOST tower enabled")
		go mpdpost.Run(ctx)
	}
	if mprisEnabled {
		slog.Info("📡 MPRISPOST tower enabled")
		go mprispost.Run(ctx)
	}

	slog.Info("📮🛰📡 Beetpost online and ready to dispatch playback events")

	quit := make(chan os.Signal, 1)
	signal.Notify(quit, syscall.SIGINT, syscall.SIGTERM)
	sig := <-quit
	slog.Info("🛑 Shutdown signal received", "signal", sig.String())

	cancel()
	slog.Info("📮 Dispatcher shut down cleanly")
}
